package vsp;

import java.util.ArrayList;

/**
 * Generates histogram
 * @author Jan Tichava <jtichava@students.zcu.cz>
 *
 */
public class Histogram {

	// Maximum marks on line
	private static final int MARKS_COUNT = 50;
	// Mark symbol
	private static final char MARK = '*';
	
	private ArrayList<Double> values;
	
	private long from;
	private long to;
	private double step;
	
	private int[] histogram;
	
	/**
	 * Create histogram
	 * 
	 * @param from histogram begin
	 * @param to histogram end
	 * @param steps number of steps
	 */
	public Histogram(Params params, int steps) {
		from = Math.round(params.getStart() - 1);
		to = Math.round(params.getC() + 1);
		
		values = new ArrayList<Double>();
		histogram = fillZeros(new int[steps + 3]);
		step = (to - from) / (steps + 0.0);
	}
	
	/**
	 * Add new value to histogram
	 * @param num new number to histogram
	 */
	public void addValue(double num) {
		values.add(num);
		
		for (int i = 0; i < histogram.length; i++) {
			if (num < (from + (i * step))) {
				histogram[i]++;
				break;
			}
		}
	}
	
	/**
	 * Fill empty arrays with zero
	 * @param array new array
	 * @return array with zeros on each index
	 */
	private int[] fillZeros(int[] array) {
		for (int i = 0; i < array.length; i++) {
			array[i] = 0;
		}
		return array;
	}
	
	/**
	 * Basic string output for development
	 */
	public String toString() {
		String str = "";
		int sum = 0;
		
		for (int i : histogram) {
			str += i + " | ";
			sum += i;
		}
		
		return str + "\n" + sum;
	}
	
	/**
	 * Print histogram to STDOUT
	 */
	public void printHistogram() {
		ArrayList<Item> items = generateHistogram();
		
		for (Item item : items) {
			for (int i = 0; i < item.getMarks(); i++) {
				System.out.print(MARK);
			}
			System.out.println("\t" + Generator.round(item.getValue()) + " (count: " + item.getCount() + ")");
		}
	}
	
	/**
	 * Generate histogram without printing
	 * @return histogram
	 */
	public ArrayList<Item> generateHistogram() {
		int max = maxValue(histogram);
		ArrayList<Item> items = new ArrayList<Item>();
		
		for (int i = 0; i < histogram.length; i++) {
			if (histogram[i] > 0) {
				items.add(new Item(histogram[i], Math.round((histogram[i] / (max + 0.0)) * MARKS_COUNT), from + (i * step) ));
			}
		}
		
		return items;
	}
	
	/**
	 * Maximum numbers from one value
	 * @param array with numbers
	 * @return maximum count
	 */
	private int maxValue(int[] array) {
		int max = 0;
		
		for (int i : array) {
			if (i > max) {
				max = i;
			}
		}
		
		return max;
	}
}
