package vsp;

/**
 * Parametrs from command line and basic operations with them
 * 
 * @author Jan Tichava <jtichava@students.zcu.cz>
 *
 */
public class Params {
	private int count;
	private double a;
	private double b;
	private double c;
	private double start = 0;
	

	/**
	 * Create new parameters
	 * 
	 * @param count count of generated numbers
	 * @param a first break
	 * @param b second break
	 * @param c last break
	 * @throws Exception
	 */
	public Params(int count, double a, double b, double c) throws Exception {
		this.count = count;
		this.a = a;
		this.b = b;
		this.c = c;
		
		validate();
	}

	/**
	 * Validate parameters
	 * @throws Exception
	 */
	private void validate() throws Exception {
		if (!(count > 0)) {
			throw new Exception("Count must be larger then 0");
		}
		
		if (!(a < b)) {
			throw new Exception("'b' must be larger than 'a'");
		}
		if (!(b < c)) {
			throw new Exception("'c' must be larger than 'b'");
		}		
	}
	
	/**
	 * Count theoretic median
	 * @return median
	 */
	public double getMedian() {
		Trapezoid trapezoid = new Trapezoid(this);
		
		double part1 = (Math.pow(a, 2) * trapezoid.getHeight())/3;
		double part2 = ((Math.pow(a, 2) - Math.pow(b, 2)) * trapezoid.getHeight())/(-2);
		double part3 = ((2 * Math.pow(b, 2) - b * c - Math.pow(c, 2)) * trapezoid.getHeight())/(-6);
		
		return part1 + part2 + part3;
	}

	/**
	 * Count theoretic deviation
	 * @return deviation
	 */
	public double getDeviation() throws Exception {
		throw new Exception("Not implemented");
	}
	
	/**
	 * Get count of generated numbers
	 * @return the count
	 */
	public int getCount() {
		return count;
	}


	/**
	 * Get first break
	 * @return the a
	 */
	public double getA() {
		return a;
	}


	/**
	 * Get second break
	 * @return the b
	 */
	public double getB() {
		return b;
	}


	/**
	 * Get last break
	 * @return the c
	 */
	public double getC() {
		return c;
	}
	
	/**
	 * Get start break
	 * @return the start
	 */
	public double getStart() {
		return start;
	}

}
