package vsp;

/**
 * Trapezoid function
 * @author Jan Tichava <jtichava@students.zcu.cz>
 *
 */
public class Trapezoid {

	// Required area of trapezoid
	private static final int AREA = 1;
	
	private Params params;
	private double height;

	/**
	 * Create new trapezoid
	 * @param params parametrs from command line
	 */
	public Trapezoid(Params params) {
		this.params = params;
		
		double top = params.getB() - params.getA();
		double bottom = params.getC();
		
		height = (2.0 * AREA) / (top + bottom);
	}
	
	/**
	 * Get function value in 'x'
	 * @param x desired value
	 * @return function value
	 */
	public double functionValue(double x) {
		return function(x);
	}
	
	/**
	 * Get function value in 'x'
	 * @param x desired value
	 * @return function value
	 */
	private double function(double x) {
		if (x < 0) {
			return 0;
		}
		else if (x < params.getA()) {
			double sinAlpha = Math.sin(height / params.getA());
			return x * sinAlpha;
		}
		else if (x < params.getB()) {
			return this.height;
		}
		else if (x < params.getC()) {
			double sinAlpha = Math.sin(height / (params.getC() - params.getB()));
			return (params.getC() - x) * sinAlpha;
		}
		else {
			return 0;
		}
	}

	/**
	 * Get trapezoid height
	 * @return the height
	 */
	public double getHeight() {
		return height;
	}
	
}
